// Just some test patterns for testing the 8x8x8 RGB cube one panel at a time. Assumes you are using one of SuperTech-IT's base boards. 

//Lights red for each column at a time from left to right, then green, then blue, then white for the given panel number (panel 0 is nearest transistors).
void fast_column_scanner(byte panel) {   //panel variable is which panel to test. 0 is the panel closest to transistors. 
  boolean backward=false; int tR=0; int tG=0; int tB=0; 
  int run=1; //specifies the number of times to scan back and forth. each color will scan 1/4 times of this number. Should be divisible by 4 preferrably.
  int x=0; //current run counter
  do  //will run for the number of iterations specified in the run variable below
   {
    x++; //increment the run counter
    //scan columns forward with red
    for (int col=0; col<8; col++) {   //loop x number of times
       for (byte lay=0; lay<8; lay++)   {   //scan through all layers and turn on all layer LEDS
          LED(col,panel,lay,255,0,0);         
       }
       delay(100); //pause with entire column lit for a bit
       clearCube();  //clear the cube
    }
    //scan columns backward with current color
    for (int col=7; col>=0; col--) {   //loop x number of times
       for (byte lay=0; lay<8; lay++)   {   //scan through all layers and turn on all layer LEDS
          LED(col,panel,lay,255,0,0);         
       }
       delay(100); //pause with entire column lit for a bit
       clearCube();  //clear the cube
    }
  } while (x < run);
  x=0; //current run counter
  do  //will run for the number of iterations specified in the run variable below
   {
    x++; //increment the run counter
    //scan columns forward with green
    for (int col=0; col<8; col++) {   //loop x number of times
       for (byte lay=0; lay<8; lay++)   {   //scan through all layers and turn on all layer LEDS
          LED(col,panel,lay,0,255,0);         
       }
       delay(100); //pause with entire column lit for a bit
       clearCube();  //clear the cube
    }
    //scan columns backward with current color
    for (int col=7; col>=0; col--) {   //loop x number of times
       for (byte lay=0; lay<8; lay++)   {   //scan through all layers and turn on all layer LEDS
          LED(col,panel,lay,0,255,0);         
       }
       delay(100); //pause with entire column lit for a bit
       clearCube();  //clear the cube
    }
  } while (x < run);
  x=0; //current run counter
  do  //will run for the number of iterations specified in the run variable below
   {
    x++; //increment the run counter
    //scan columns forward with Blue
    for (int col=0; col<8; col++) {   //loop x number of times
       for (byte lay=0; lay<8; lay++)   {   //scan through all layers and turn on all layer LEDS
          LED(col,panel,lay,0,0,255);         
       }
       delay(100); //pause with entire column lit for a bit
       clearCube();  //clear the cube
    }
    //scan columns backward with current color
    for (int col=7; col>=0; col--) {   //loop x number of times
       for (byte lay=0; lay<8; lay++)   {   //scan through all layers and turn on all layer LEDS
          LED(col,panel,lay,0,0,255);         
       }
       delay(100); //pause with entire column lit for a bit
       clearCube();  //clear the cube
    }
  } while (x < run);
  x=0; //current run counter
  do  //will run for the number of iterations specified in the run variable below
   {
    x++; //increment the run counter
    //scan columns forward with white
    for (int col=0; col<8; col++) {   //loop x number of times
       for (byte lay=0; lay<8; lay++)   {   //scan through all layers and turn on all layer LEDS
          LED(col,panel,lay,255,255,255);         
       }
       delay(100); //pause with entire column lit for a bit
       clearCube();  //clear the cube
    }
    //scan columns backward with current color
    for (int col=7; col>=0; col--) {   //loop x number of times
       for (byte lay=0; lay<8; lay++)   {   //scan through all layers and turn on all layer LEDS
          LED(col,panel,lay,255,255,255);         
       }
       delay(100); //pause with entire column lit for a bit
       clearCube();  //clear the cube
    }
  } while (x < run);
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// Cycles each column through Red, then Green, then Blue for the given panel number (panel 0 is nearest transistors).
void column_by_column(byte panel) {   //panel variable is which panel to test.
  for (byte column=0; column<8; column++) {   //Turn on all LEDs in each column 1 at a time
     for (byte layer=0; layer<8; layer++)   {   //scan through all layers in this column turning on LEDS
          LED(column,panel,layer,255,0,0); //turn on red for five seconds
     }
     delay(500);
     for (byte layer=0; layer<8; layer++)   {   //scan through all layers in this column turning on LEDS
          LED(column,panel,layer,0,255,0); //turn on Green for five seconds
     }
     delay(500);
     for (byte layer=0; layer<8; layer++)   {   //scan through all layers in this column turning on LEDS
          LED(column,panel,layer,0,0,255); //turn on blue for five seconds
     }
     delay(500);
     for (byte layer=0; layer<8; layer++)   {   //scan through all layers in this column turning on LEDS
          LED(column,panel,layer,0,0,0); //turn off all leds in column
     }
   }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/* Lights red for each LED starting at bottom left, chasing through each layer working up to the top for the given panel number. 
  Then repeats with Green, then Blue, then White (panel 0 is nearest transistors). */
  
 void one_by_one(byte panel) {
   //red
  for (byte layer=0; layer<8; layer++) {   //turn on one layer at a time
      for (byte column=0; column<8; column++) {   //turn on each of the LEDS in a column 1 by 1)
         LED(column,panel,layer,255,0,0);  //turn on red for two seconds
         delay(100);
         LED(column,panel,layer,0,0,0); //turn off led before going to the next
      }
  }
  //green
  for (byte layer=0; layer<8; layer++) {   //turn on one layer at a time
      for (byte column=0; column<8; column++) {   //turn on each of the LEDS in a column 1 by 1)
         LED(column,panel,layer,0,255,0);  //turn on red for two seconds
         delay(100);
         LED(column,panel,layer,0,0,0); //turn off led before going to the next
      }
  }
  //blue
  for (byte layer=0; layer<8; layer++) {   //turn on one layer at a time
      for (byte column=0; column<8; column++) {   //turn on each of the LEDS in a column 1 by 1)
         LED(column,panel,layer,0,0,255);  //turn on red for two seconds
         delay(100);
         LED(column,panel,layer,0,0,0); //turn off led before going to the next
      }
  }
  //white
  for (byte layer=0; layer<8; layer++) {   //turn on one layer at a time
      for (byte column=0; column<8; column++) {   //turn on each of the LEDS in a column 1 by 1)
         LED(column,panel,layer,255,255,255);  //turn on red for two seconds
         delay(100);
         LED(column,panel,layer,0,0,0); //turn off led before going to the next
      }
  }
}  

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// Lights the given panel in solid red, then green, then blue, then white (panel 0 is nearest transistors).
void whole_panel(byte panel){
  //turn on all red
  for (byte layer=0; layer<8; layer++){  // scan thru each layer
    for (byte column=0; column<8; column++){  // scan thru every column
        cube[layer][panel][column][0]=255;
        cube[layer][panel][column][1]=0;
        cube[layer][panel][column][2]=0;
    }
  }
  delay(5000);
  //turn on all green
  for (byte layer=0; layer<8; layer++){  // scan thru each layer
    for (byte column=0; column<8; column++){  // scan thru every column
        cube[layer][panel][column][0]=0;
        cube[layer][panel][column][1]=255;
        cube[layer][panel][column][2]=0;
    }
  }
  delay(5000);
  //turn on all blue
  for (byte layer=0; layer<8; layer++){  // scan thru each layer
    for (byte column=0; column<8; column++){  // scan thru every column
        cube[layer][panel][column][0]=0;
        cube[layer][panel][column][1]=0;
        cube[layer][panel][column][2]=255;
    }
  }
  delay(5000);
  //turn on all white
  for (byte layer=0; layer<8; layer++){  // scan thru each layer
    for (byte column=0; column<8; column++){  // scan thru every column
        cube[layer][panel][column][0]=255;
        cube[layer][panel][column][1]=255;
        cube[layer][panel][column][2]=255;
    }
  }
  delay(5000);
  clearCube();
}

