/* There are currently six subroutines here:
 1) The LED sub provides the ability to light any particular LED at a specified location with specified 3 RGB color components. 
 2) The getColor sub the selects colors of the rainbow by number and intensity
 3) The clearCube sub that clears the cube and turn all LEDs off. 
 4) The scrollText sub lets you specify a string of text to be scrolled, specify its 3 RGB color conponents, and whether to display on 2 or 4 faces of the cube. 
 5) The scrollTextStd2 lets you specify a string and standard color for display on 2 faces of the cube
 6) The scrollTextStd4 lets you specify a string and standard color for display on 4 faces of the cube
 
 
 
/* First, thanks to SuperTech-IT for suggesting this subroutine. 
 Very straight forward.  You specify a particular LED with x,y,and z.  Then specify the color by specifying the 3 RGB color components.
 This routine then loads it into the cube matrix. Coordinates are assuming you are looking at the back of the cube. X is left right, Y is up down, Z in in out. 
 */

void LED(int x,int y,int z,int R,int G,int B) {
  if (R>63){
    R=63;
  }
  if (G>63){
    G=63;
  }
  if (B>63){
    B=63;
  }
  cube[x][y][z][0]=R;
  cube[x][y][z][1]=G;
  cube[x][y][z][2]=B;
}




      


/* getColor is a subroutine to get 189 colors of the rainbow. It starts at 0 for Red and steps through yellows, greens, aguas, blues, 
 purples,etc. ending at 189.  190 specifies white, and anything above that specifies black (all off).   Intensities are 1 through 4 with
 4 the brightest.  0 intensity is all off.  There's no color wheel math here - just two colors mixed with a combined intensity = 63. Then
 for lower intensities, with each step down in intensity, all three color intensities are divided by 2. 
 
 A few standard colors my be specified by their names: Black, Red, Orange, Yellow, Green, Aqua, Blue, Violet, Purple,
 and White.  Intensities for these standard colors are also 0 (full off) to 4 (brightest).
 
 It sets up the RGB color components for the selected color in global variables myred, mygreen, and myblue. These can then be transferred to the cube array.  
 */
void getColor(int color, int intensity) {

  if (color<64){
    mycase=1;
  }
  if ((color>63)&& (color<127)){
    mycase=2;
  }
  if ((color>126)&& (color<190)){
    mycase=3;
  }
  if (color==190){
    mycase=4;
  }
  if (color>190){
    mycase=5;
  }
  switch(mycase){     
  case 1:  
    mygreen = color;
    myred = 63-mygreen;
    myblue = 0;
    break; 
  case 2:  
    myblue = color-63;
    mygreen = 63-myblue;
    myred = 0;
    break; 
  case 3: 
    myred = color-127;
    myblue = 63-myred;
    mygreen = 0;
    break; 
  case 4: 
    myred = 63;
    myblue = 63;
    mygreen = 63;
    break; 
  case 5: 
    myred = 0;
    myblue = 0;
    mygreen = 0;
    break; 
  }
  if (intensity <5){
    myred=myred/2;
    mygreen=mygreen/2;
    myblue=myblue/2;
  }
  if (intensity <4){
    myred=myred/2;
    mygreen=mygreen/2;
    myblue=myblue/2;
  }
  if (intensity <3){
    myred=myred/2;
    mygreen=mygreen/2;
    myblue=myblue/2;
  }
  if (intensity <2){
    myred=myred/2;
    mygreen=mygreen/2;
    myblue=myblue/2;
  }
  if (intensity <1){
    myred=0;
    mygreen=0;
    myblue=0;
  }
}

// This subroutine turns off every LED in the cube.
void clearCube(){
  for (byte layer=0; layer<8; layer++){  // scan thru each layer
    for (byte column=0; column<8; column++){  // scan thru every column
      for (byte panel=0; panel<8; panel++){  // scan thru every panel
        cube[layer][panel][column][0]=0;
        cube[layer][panel][column][1]=0;
        cube[layer][panel][column][2]=0;
      }
    }
  }
}

/* This subroutine scrolls text around the outside walls of the cube.  You first specify a string of text up to 120 characters long. 
 Then specify its color as red, green, blue components.  Finally specify Mode, which is 2 for scolling on only 2 walls of the 
 or 4 for scrolling on all 4 walls. Speed of scrolling is defined by scrollRate in the first tab.  
 */
void scrollText(char mystring[120], int xRed, int xGreen, int xBlue, int Mode){
  for (int xxx=0; xxx<120; xxx++){ // This "for" loop processes the incoming string one character at a time
    incomingByte= mystring[xxx];
    if (incomingByte == '\0'){ // if end of string is found,.
      for (int count=0; count<4; count++){ // scroll the last character off the cube
        scrollOneChar(xRed, xGreen, xBlue, Mode);
      }
      delay(1000);  // wait 1 second and exit the loop
      break;
    }
    offset= (incomingByte-32)*8;  // convert one character in the string into font code
    for (int col=0; col<8; col++){
      for (int row=0; row<8; row++){
        if (pgm_read_byte_near(&myfont[offset+col][row])>0) {
          text_buffer[col+37][row]=1;  // and put it into the text buffer in the last position.
        }
      }
    }
    // This moves the characters around the cube,   It's in behind the scenes. 
    scrollOneChar(xRed, xGreen, xBlue, Mode);  
  }
  for (int col=0; col<46; col++){  // when we are all finished with this string, clear the text buffer. 
    for (int row=0; row<8; row++){
      text_buffer[col][row]=0;
    }
  }
}

// This is a simplified text scrolling routine that require only an input string and a specified standard 
// color like Red, Orange, White, etc.  Scrolls text on 2 sides of cube. 
void scrollTextStd2(char mystring[120], int xColor){
  getColor(xColor, 4);
  scrollText(mystring, myred, mygreen, myblue, 2);
}

// This is a simplified text scrolling routine that require only an input string and a specified standard 
// color like Red, Orange, White, etc.  Scrolls text on 4 sides of cube. 
void scrollTextStd4(char mystring[120], int xColor){
  getColor(xColor, 4);
  scrollText(mystring, myred, mygreen, myblue, 4);
}

void zdown (int times) {
  //shift entire cube down (times) layer(s)
  for (int i=0;i<times;i++) {
    for (int x=0;x<8;x++){
      for (int y=0;y<8;y++){
        for (int z=1;z<8;z++){
          for (int c=0;c<3;c++){
            cube [x][y][z-1][c] = cube [x][y][z][c];
          }
        }
      }
    }
  }
}






