/*  These are items that you probably don't want to mess with unless you really understand them.
    There are currently 2 items here:  
  1) the timer interupt routine to refresh the cube
  2) the text scrolling core which advances text from one cube face to the next


This timer interrupt refreshes the whole cube, based on data stored in the cube matrix.  It uses 6 bit BAM (bit angle modulation)
 to set the levels of RGB stored in the cube matrix.
 Within the interrupt routine, all output is done using direct port toggling. With this approach, all 8 layers are refreshed in under 6 msec.  
 The LEDs in each layer (if set for max brightness) are on for .64 msec.(the sum of the BAM times).  This interrupt routine is called every 
 8 msec., so the LEDs are on 64% of theoretical best (which would be 1 msec. for each of the 8 layers). 
 */
uint32_t refreshCube(uint32_t currentTime) {
  byte red, green, blue;
  for (byte count=0; count<6; count++){  // BAM counter; each increment doubles the time the LED is on, starting at 5 microsecs.
    digitalWrite(Latch, LOW );  //make sure outputs are latched
    LATDCLR = CLK|SDIR|SDIG|SDIB;
    for (byte layer=0; layer<8; layer++){  // scan thru each layer
      for (byte column=0; column<8; column++){  // scan thru every column
        for (byte panels=8; panels>0; panels--){  // scan thru every panel FIX HERE FOR BACKWARD TEXT
        byte panel = panels-1; // FIX HERE FOR BACKWARD TEXT
          red = cube[column][panel][layer][0]; // get its red component
          if ((red & 1<<count)>0) {  // BAM it and shift it out

            LATDSET |= SDIR;
          }
          green = cube[column][panel][layer][1];  // get its green component
          if ((green & 1<<count)>0) {  // BAM it and shift it out
            LATDSET |= SDIG;
          }
          blue = cube[column][panel][layer][2];  // get its blue component
          if ((blue & 1<<count)>0) {  // BAM it and shift it out
            LATDSET |= SDIB;
          }
          LATDSET = CLK;  //Clock the data in.
          _nop(); 
          LATDCLR = CLK|SDIR|SDIG|SDIB;	
        }
      }
      LATDSET = LE;  // latch the data
      _nop(); 
      LATDCLR = LE;
      LATECLR = LAYER[layer]; // turn on layer
      delayMicroseconds((1<<count)*10);  //here count sets time the layer is on - starting with 10 microsecs and ending at 32*10 or 320 microsecs.
      LATESET = LAYER[layer]; //turn off layer
    }
  }
  return (currentTime + CORE_TICK_RATE*8);
}

//  This subroutine scrolls the characters being displayed from one cube face to the next.  
//  It is pretty much the guts of text display, and probably should be left alone.   
void scrollOneChar(int xRed, int xGreen, int xBlue, int Mode){
  for (int shift=0; shift<8; shift++) { // Before going to the next character, display the content of the text buffer 
    for (int col=0; col<45; col++){     // eight times, each time shifting the content of the text buffer to the left by one LED.
      for (int row=0; row<8; row++){
        text_buffer[col][row]=text_buffer[col+1][row];
      }
    }
    for (int col=0; col<8; col++){  // Here the text buffer is actually displayed across the 4 walls of the cube 
      for (int row=0; row<8; row++){
        if ((text_buffer[col+29][row]>0) && (Mode==4)){  // Note Mode must be 4 for this wall to display
          LED(7-col, 7, row, xRed, xGreen, xBlue); 
        }
        if (text_buffer[col+22][row]>0){
          LED(7, col, row, xRed, xGreen, xBlue); 
        }
        if (text_buffer[col+15][row]>0){
          LED(col, 0, row, xRed, xGreen, xBlue); 
        }
        if ((text_buffer[col+8][row]>0) && (Mode==4)){  // Note Mode must be 4 for this wall to display
          LED(0, 7-col, row, xRed, xGreen, xBlue); 
        }
      }
    }
    delay(scrollRate/8); // Wait 100 msec.  before indexing display over one LED to the left
    for (int col=0; col<8; col++){  // Clear all 4 walls before re-writing text buffer to LEDs
      for (int row=0; row<8; row++){
        LED(col, 7, row, 0, 0, 0); 
        LED(7, col, row, 0, 0, 0); 
        LED(col, 0, row, 0, 0, 0); 
        LED(0, col, row, 0, 0, 0); 
      }
    }
  }
}


